@@ -4,11 +4,12 @@
 # Copyright Tumult Labs 2022
 
 import datetime
+import functools
 import warnings
 from abc import ABC, abstractmethod
 from collections import OrderedDict
 from dataclasses import dataclass
-from typing import Any, Dict, Optional, Sequence
+from typing import Any, Dict, Mapping, Optional, Sequence
 
 import numpy as np
 from pyspark import Row
@@ -36,6 +37,7 @@ from tmlt.core.domains.numpy_domains import (
 )
 from tmlt.core.domains.pandas_domains import PandasDataFrameDomain
 from tmlt.core.utils.grouped_dataframe import GroupedDataFrame
+from tmlt.core.utils.misc import get_nonconflicting_string
 
 
 class SparkColumnDescriptor(ABC):
@@ -518,8 +520,35 @@ class SparkGroupedDataFrameDomain(Domain):
                 f"Expected: {self.group_keys.schema}. Got: {value.group_keys.schema}"
             )
 
+        if not self.group_keys.columns:
+            # Other checks are not necessary if there are no columns.
+            return
+
+        # Note that we are using an inner join instead of intersect
+        # This is b/c intersect sometimes drops rows for unknown reasons
+        # see https://issues.apache.org/jira/browse/SPARK-40181
+        # We are using column aliases to prevent Spark from raising a warning when
+        # identical domains are being compared.
+        aliases = functools.reduce(
+            lambda all_keys, _: all_keys + [get_nonconflicting_string(all_keys)],
+            self.group_keys.columns,
+            self.group_keys.columns,
+        )[len(self.group_keys.columns) :]
+        intersection = self.group_keys.join(
+            value.group_keys,
+            on=[
+                self.group_keys[column].eqNullSafe(
+                    value.group_keys[column].alias(aliases[idx])
+                )
+                for idx, column in enumerate(self.group_keys.columns)
+            ],
+            how="inner",
+        )
+        # this join duplicates columns, drop the duplicates
+        for column in value.group_keys.columns:
+            intersection = intersection.drop(value.group_keys[column])
         invalid_group_keys = self.group_keys.union(value.group_keys).subtract(
-            self.group_keys.intersect(value.group_keys)
+            intersection
         )
         if invalid_group_keys.first():
             raise OutOfDomainError("Groups keys do not match")
@@ -559,16 +588,36 @@ class SparkGroupedDataFrameDomain(Domain):
             self.group_keys.schema != other.group_keys.schema
         ):
             return False
+        if not self.group_keys.columns:
+            return True
         group_keys_count = self.group_keys.count()
         if group_keys_count != other.group_keys.count():
             return False
-        # This only works because group_keys contains distinct rows.
-        # intersecting two empty dataframes produces a row!
-        return self.group_keys.intersect(
-            other.group_keys
-        ).count() == group_keys_count or (
-            group_keys_count == 0 and other.group_keys.count() == 0
+        # Note that we are using an inner join instead of intersect
+        # that is b/c intersect sometimes drops rows for unknown reasons
+        # see https://issues.apache.org/jira/browse/SPARK-40181
+
+        # We are using column aliases to prevent Spark from raising a warning when
+        # identical domains are being compared.
+        aliases = functools.reduce(
+            lambda all_keys, _: all_keys + [get_nonconflicting_string(all_keys)],
+            self.group_keys.columns,
+            self.group_keys.columns,
+        )[len(self.group_keys.columns) :]
+        intersection = self.group_keys.join(
+            other.group_keys,
+            on=[
+                self.group_keys[column].eqNullSafe(
+                    other.group_keys[column].alias(aliases[idx])
+                )
+                for idx, column in enumerate(self.group_keys.columns)
+            ],
+            how="inner",
         )
+        # this join duplicates columns, drop the duplicates
+        for column in other.group_keys.columns:
+            intersection = intersection.drop(other.group_keys[column])
+        return intersection.count() == group_keys_count
 
     def __getitem__(self, col_name: str) -> SparkColumnDescriptor:
         """Returns column descriptor for given column."""
@@ -577,7 +626,7 @@ class SparkGroupedDataFrameDomain(Domain):
 
 def convert_spark_schema(spark_schema: StructType) -> SparkColumnsDescriptor:
     """Returns mapping from column name to SparkColumnDescriptor."""
-    spark_type_to_size = {
+    spark_type_to_size: Mapping[DataType, int] = {
         IntegerType(): 32,
         LongType(): 64,
         FloatType(): 32,
@@ -585,10 +634,10 @@ def convert_spark_schema(spark_schema: StructType) -> SparkColumnsDescriptor:
     }
     column_to_descriptor = dict()
     for field in spark_schema:
-        if field.name in spark_schema:
+        if field.name in column_to_descriptor:
             raise ValueError(f"Schema contains duplicate column name {field.name}.")
         column_to_descriptor[field.name] = _spark_type_to_descriptor(
-            field.dataType, field.nullable, spark_type_to_size.get(field.dataType, None)
+            field.dataType, field.nullable, spark_type_to_size.get(field.dataType)
         )
 
     return column_to_descriptor
